//*****************************************************************************
//
// Author		: Konstantin Chizhov
// Date			: 2013
// All rights reserved.

// Redistribution and use in source and binary forms, with or without modification, 
// are permitted prOVided that the following conditions are met:
// Redistributions of source code must retain the abOVe copyright notice, 
// this list of conditions and the following disclaimer.

// Redistributions in binary form must reproduce the abOVe copyright notice, 
// this list of conditions and the following disclaimer in the dOCumentation and/or 
// other materials prOVided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
// IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
// OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

//********************************************************************************
// ATmega164A/PA/324A/PA/644A/PA/1284*/P               HAS TIMER D,D,H,H*
//********************************************************************************
// ATmega48A-48PA-88A-88PA-168A-168PA-328-328P         HAS TIMER D,D,H
//********************************************************************************
	#pragma once

	#include <stdint.h>
	#include <stddef.h>

	#include <avr/io.h>
	#include <avr/interrupt.h>

	#include <ioreg.h>
//#include <static_assert.h>
//#include <enum.h>             // DECLARE_ENUM_OPERATIONS() //.._TEMPLATE() ??
//#include <clock.h>
//#include <data_transfer.h>    // TransferCallback ??

	#include <basicTimerD_local.h>      // typedef BaseTimerD<Timer0/2Regs> Timer0/2;
	#include <basicTimerH_local.h>      // typedef BaseTimerH<Timer1/3Regs> Timer1/3;


	namespace Mcucpp { namespace Timers {
//*****************************************************************************

	struct Timer0Regs {
//*************************************************
		IO_REG_WRAPPER(TIMSK0, IntMask,   uint8_t);  // Interrupt Mask
		IO_REG_WRAPPER(TIFR0,  IntFlag,   uint8_t);  // Interrupt Flag
		IO_REG_WRAPPER(TCCR0A, ControlA,  uint8_t);  // Counter Control|A|
		IO_REG_WRAPPER(TCCR0B, ControlB,  uint8_t);  //                |B|
		IO_REG_WRAPPER(TCNT0 , Counter,   uint8_t);  // Counter        | |
		IO_REG_WRAPPER(OCR0A,  OCompareA, uint8_t);  // Output Compare |A|
		IO_REG_WRAPPER(OCR0B,  OCompareB, uint8_t);  // Output Compare |B|
	
		enum {
	//**************************
			TOIE_  = 1 << TOIE0,  // TIMSK0 // Interrupt Mask
			OCIE_A = 1 << OCIE0A,
			OCIE_B = 1 << OCIE0B,

			TOV_   = 1 << TOV0,   // TIFR0 // Interrupt Flag
			OCF_A  = 1 << OCF0A,
			OCF_B  = 1 << OCF0B,

			WGM_0  = 1 << WGM00,  // TCCR0A // Counter Control|A|
			WGM_1  = 1 << WGM00,
			COM_B0 = 1 << COM0B0,
			COM_B1 = 1 << COM0B1,
			COM_A0 = 1 << COM0A0,
			COM_A1 = 1 << COM0A1,

			CS_0   = 1 << CS00,   // TCCR0B  // Counter Control|B|
			CS_1   = 1 << CS01,
			CS_2   = 1 << CS02,
			WGM_2  = 1 << WGM02,

			FOC_B  = 1 << FOC0B,  // TCCR0C  // Counter Control|C|
			FOC_A  = 1 << FOC0A,
		};

	}; //.struct Timer0Regs

	#define MCUCPP_HAS_TIMER0
//****************************************
	typedef BaseTimerD<Timer0Regs> Timer0;
//****************************************


	struct Timer2Regs {
//*************************************************
		IO_REG_WRAPPER(TIMSK2, IntMask,   uint8_t);  // Interrupt Mask
		IO_REG_WRAPPER(TIFR2,  IntFlag,   uint8_t);  // Interrupt Flag
		IO_REG_WRAPPER(TCCR2A, ControlA,  uint8_t);  // Counter Control|A|
		IO_REG_WRAPPER(TCCR2B, ControlB,  uint8_t);  //                |B|
		IO_REG_WRAPPER(TCNT2 , Counter,   uint8_t);  // Counter        | |
		IO_REG_WRAPPER(OCR2A,  OCompareA, uint8_t);  // Output Compare |A|
		IO_REG_WRAPPER(OCR2B,  OCompareB, uint8_t);  // Output Compare |B|
	
		enum {
	//**************************
			TOIE_  = 1 << TOIE2,  // TIMSK2 // Interrupt Mask
			OCIE_A = 1 << OCIE2A,
			OCIE_B = 1 << OCIE2B,

			TOV_   = 1 << TOV2,   // TIFR2 // Interrupt Flag
			OCF_A  = 1 << OCF2A,
			OCF_B  = 1 << OCF2B,

			WGM_0  = 1 << WGM20,  // TCCR2A // Counter Control|A|
			WGM_1  = 1 << WGM20,
			COM_B0 = 1 << COM2B0,
			COM_B1 = 1 << COM2B1,
			COM_A0 = 1 << COM2A0,
			COM_A1 = 1 << COM2A1,

			CS_0   = 1 << CS20,   // TCCR2B  // Counter Control|B|
			CS_1   = 1 << CS21,
			CS_2   = 1 << CS22,
			WGM_2  = 1 << WGM22,

			FOC_B  = 1 << FOC2B,  // TCCR2C  // Counter Control|C|
			FOC_A  = 1 << FOC2A,
		};

	}; //.struct Timer2Regs
	
	#define MCUCPP_HAS_TIMER2
//****************************************
	typedef BaseTimerD<Timer2Regs> Timer2;
//****************************************

	struct Timer1Regs {
//*************************************************
		IO_REG_WRAPPER(TIMSK1, IntMask,   uint8_t);  // Interrupt Mask
		IO_REG_WRAPPER(TIFR1,  IntFlag,   uint8_t);  // Interrupt Flag
		IO_REG_WRAPPER(TCCR1A, ControlA,  uint8_t);  // Counter Control|A|
		IO_REG_WRAPPER(TCCR1B, ControlB,  uint8_t);  //                |B|
		IO_REG_WRAPPER(TCCR1C, ControlC,  uint8_t);  //                |C|
		IO_REG_WRAPPER(TCNT1 , Counter,   uint16_t); // Counter        | |   16 Bit
		IO_REG_WRAPPER(OCR1A,  OCompareA, uint16_t); // Output Compare |A|   16 Bit 
		IO_REG_WRAPPER(OCR1B,  OCompareB, uint16_t); // Output Compare |B|   16 Bit
		IO_REG_WRAPPER(ICR1,   ICompare,  uint16_t); // Input  Compare | |   16 Bit
//*************************************************
//	IO_REG_WRAPPER(TCNT1H, Tcnt1H,    uint8_t ); // Counter        | |H|  8 Bit 
//	IO_REG_WRAPPER(TCNT1L, Tcnt1L,    uint8_t ); //                | |L|  8 Bit 
//	IO_REG_WRAPPER(OCR1AH, Ocr1aH,    uint8_t ); // Output Compare |A|H|  8 Bit
//	IO_REG_WRAPPER(OCR1AL, Ocr1aL,    uint8_t ); //                |A|L|  8 Bit 
//	IO_REG_WRAPPER(OCR1BH, Ocr1bH,    uint8_t ); // Output Compare |B|H|  8 Bit
//	IO_REG_WRAPPER(OCR1BL, Ocr1bL,    uint8_t ); // Output Compare |B|L|  8 Bit
//	IO_REG_WRAPPER(ICR1H,  Icr1H,     uint8_t ); // Input  Compare | |H|  8 Bit
//	IO_REG_WRAPPER(ICR1L,  Icr1L,     uint8_t ); // Input  Compare | |L|  8 Bit

		enum {
	//**************************
			TOIE_  = 1 << TOIE1,  // TIMSK1 // Interrupt Mask
			OCIE_A = 1 << OCIE1A,
			OCIE_B = 1 << OCIE1B,
			ICIE_  = 1 << ICIE1,

			TOV_   = 1 << TOV1,   // TIFR1 // Interrupt Flag
			OCF_A  = 1 << OCF1A,
			OCF_B  = 1 << OCF1B,
			ICF_   = 1 << ICF1,

			WGM_0  = 1 << WGM10,  // TCCR1A // Counter Control|A|
			WGM_1  = 1 << WGM11,
			COM_B0 = 1 << COM1B0,
			COM_B1 = 1 << COM1B1,
			COM_A0 = 1 << COM1A0,
			COM_A1 = 1 << COM1A1,

			CS_0   = 1 << CS10,   // TCCR1B  // Counter Control|B|
			CS_1   = 1 << CS11,
			CS_2   = 1 << CS12,
			WGM_2  = 1 << WGM12,
			WGM_3  = 1 << WGM13,
			ICES_  = 1 << ICES1,
			ICEN_  = 1 << ICNC1,

			FOC_B  = 1 << FOC1B,  // TCCR1C  // Counter Control|C|
			FOC_A  = 1 << FOC1A,
		};
	}; //.struct Timer1Regs

	#define MCUCPP_HAS_TIMER1
//****************************************
	typedef BaseTimerH<Timer1Regs> Timer1;
//****************************************

	#if defined (__AVR_ATmega1284__) | \
			defined (__AVR_ATmega1284P__)
//*************************************************
	struct Timer3Regs {
//*************************************************
		IO_REG_WRAPPER(TIMSK3, IntMask,   uint8_t);  // Interrupt Mask
		IO_REG_WRAPPER(TIFR3,  IntFlag,   uint8_t);  // Interrupt Flag
		IO_REG_WRAPPER(TCCR3A, ControlA,  uint8_t);  // Counter Control|A|
		IO_REG_WRAPPER(TCCR3B, ControlB,  uint8_t);  //                |B|
		IO_REG_WRAPPER(TCCR3C, ControlC,  uint8_t);  //                |C|
		IO_REG_WRAPPER(TCNT3 , Counter,   uint16_t); // Counter        | |   16 Bit
		IO_REG_WRAPPER(OCR3A,  OCompareA, uint16_t); // Output Compare |A|   16 Bit 
		IO_REG_WRAPPER(OCR3B,  OCompareB, uint16_t); // Output Compare |B|   16 Bit
		IO_REG_WRAPPER(ICR3,   ICompare,  uint16_t); // Input  Compare | |   16 Bit

		enum {
	//**************************
			TOIE_  = 1 << TOIE3,  // TIMSK3 // Interrupt Mask
			OCIE_A = 1 << OCIE3A,
			OCIE_B = 1 << OCIE3B,
			ICIE_  = 1 << ICIE3,

			TOV_   = 1 << TOV3,   // TIFR3 // Interrupt Flag
			OCF_A  = 1 << OCF3A,
			OCF_B  = 1 << OCF3B,
			ICF_   = 1 << ICF3,

			WGM_0  = 1 << WGM30,  // TCCR3A // Counter Control|A|
			WGM_1  = 1 << WGM31,
			COM_B0 = 1 << COM3B0,
			COM_B1 = 1 << COM3B1,
			COM_A0 = 1 << COM3A0,
			COM_A1 = 1 << COM3A1,

			CS_0   = 1 << CS30,   // TCCR3B  // Counter Control|B|
			CS_1   = 1 << CS31,
			CS_2   = 1 << CS32,
			WGM_2  = 1 << WGM32,
			WGM_3  = 1 << WGM33,
			ICES_  = 1 << ICES3,
			ICEN_  = 1 << ICNC3,

			FOC_B  = 1 << FOC3B,  // TCCR3C  // Counter Control|C|
			FOC_A  = 1 << FOC3A,
		};
	}; //.struct Timer1Regs

	#define MCUCPP_HAS_TIMER3
//****************************************
	typedef BaseTimerH<Timer3Regs> Timer3;
//****************************************
	#endif


//________________________________________________________________________________
}} //.Timers //.Mcucpp                                               a.b.08.2015



// Code Snippet                                                         /ioreg.h
/*********************************************************************************
#define IO_REG_WRAPPER(REG_NAME, CLASS_NAME, DATA_TYPE) \
	struct CLASS_NAME {\
\
		typedef DATA_TYPE DataT;\
\
		static DataT Get  (           ){ return         REG_NAME;         }\
		static void  Set  (DataT value){                REG_NAME  = value;}\
		static void  Or   (DataT value){                REG_NAME |= value;}\
		static void  And  (DataT value){                REG_NAME &= value;}\
		static void  Xor  (DataT value){                REG_NAME ^= value;}\
		static void  AndOr(DataT andMask, DataT orMask){REG_NAME  =(REG_NAME & andMask) | orMask;}\
\
		template<int Bit>\
		static bool  BitIsSet  (){return   REG_NAME & (1 << Bit);}\
\
		template<int Bit>\
		static bool  BitIsClear(){return !(REG_NAME & (1 << Bit));}\
	}

**********************************************************************************/
