//*********************************************************************************
//
// Author		: Konstantin ChizhOV
// Date			: 2013
// All rights reserved.

// Redistribution and use in source and binary forms, with or without modification, 
// are permitted prOVided that the following conditions are met:
// Redistributions of source code must retain the abOVe copyright notice, 
// this list of conditions and the following disclaimer.

// Redistributions in binary form must reproduce the abOVe copyright notice, 
// this list of conditions and the following disclaimer in the documentation and/or 
// other materials prOVided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
// IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
// OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//*********************************************************************************
	#pragma once
	
	namespace Mcucpp { namespace Timers {
//*********************************************************************************

	template < class Regs // register definitions 
//************************
>	class BaseTimerD {
//*********************************************************************************
	public:
		typedef uint8_t DataT;

		enum { 
	//*******************************************************************************
			MaxValue                      =    0xff 
		};

		enum InterruptFlags {
	//*******************************************************************************
			NoInterrupt                   =    0,
			OverflowInt                   =    Regs::TOIE_,
			OutCompare0AInt               =    Regs::OCIE_A,
			OutCompare0BInt               =    Regs::OCIE_B,
		};

		enum TimerMode {
	//*******************************************************************************
			Normal                        =    0,
			PwmPhaseCorrect0xFF           =                           Regs::WGM_0,
			ClearOnCompare                =                Regs::WGM_1, 
			FastPwm0xFF                   =                Regs::WGM_1|Regs::WGM_0,
		//Reserved                      =    Regs::WGM_2
			PwmPhaseCorrectORCA           =    Regs::WGM_2|            Regs::WGM_0,
		//Reserved_                     =    Regs::WGM_2|Regs::WGM_1,
			PwmFastORCA                   =    Regs::WGM_2|Regs::WGM_1|Regs::WGM_0,
		};


		enum ClockDivider {
	//*******************************************************************************
			DivStop                       =    0, 
			Div1                          =                          Regs::CS_0, 
			Div8                          =               Regs::CS_0, 
			Div64                         =               Regs::CS_0|Regs::CS_0, 
			Div256                        =    Regs::CS_2, 
			Div1024                       =    Regs::CS_2|           Regs::CS_0,
			ExtFalling                    =    Regs::CS_2|Regs::CS_0, 
			ExtRising                     =    Regs::CS_2|Regs::CS_0|Regs::CS_0
		};

		enum { ClockDivider_ClearMask    = ~(Regs::CS_2|Regs::CS_1 |Regs::CS_0  )};
		enum { ModeControlA_Mask         =  (           Regs::WGM_0|Regs::WGM_1 )};
		enum { ModeControlB_Mask         =  (           Regs::WGM_2             )};
		enum { ModeControlA_ClearMask    = ~(           Regs::WGM_0|Regs::WGM_1 )};
		enum { ModeControlB_ClearMask    = ~(           Regs::WGM_2             )};


		static DataT Get                ( ){return Regs::Counter ::Get()           ;};
		static void  Set        (DataT val){       Regs::Counter ::Set(val)        ;};
		static void  Stop               ( ){       Regs::ControlB::Set(0)          ;};
		static void  Clear              ( ){       Regs::Counter ::Set(0)          ;};
		static void  EnableInterrupt    ( ){       Regs::IntMask ::Or (Regs::TOIE_);};
		static bool  IsInterrupt        ( ){return Regs::IntMask ::And(Regs::TOV_ );};
		static void  ClearInterruptFlag ( ){       Regs::IntMask ::Or (Regs::TOV_ );};

		static void  Start (ClockDivider divider) {
	//*********************************************************
	//ControlRegB = (ControlRegB & ClockDividerMask) | divider;
	//*********************************************************
			Regs::ControlB::AndOr(ClockDivider_ClearMask,divider); 
		}
		static void SetMode(TimerMode mode) {
	//******************************************************************************
	//ControlRegA = (ControlRegA & "TimerMode"ClearMaskA) | mode;
	//ControlRegA = (ControlRegB & "TimerMode"ClearMaskA) | mode;
	//""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
			Regs::ControlA::AndOr(ModeControlA_ClearMask, mode );
			Regs::ControlB::AndOr(ModeControlB_ClearMask, mode );
		}

//	template<unsigned u> struct Divider;                                   // ??

		class OutputCompareA{
	// *****************************************************************************
		public:
			static void  Set      (DataT val){       Regs::OCompareA::Set(val)         ;};
			static DataT Get               (){return Regs::OCompareA::Get()            ;};
			static void  EnableInterrupt   (){       Regs::IntMask  ::Or (Regs::OCIE_A);};
			static bool  IsInterrupt       (){return Regs::IntFlag  ::And(Regs::OCIE_A);};
			static void  ClearInterruptFlag(){       Regs::IntFlag  ::Or (Regs::OCF_A );};
		};
		

		class OutputCompareB {
	// *****************************************************************************
		public:
			static void  Set      (DataT val){       Regs::OCompareB::Set(val)         ;}
			static DataT Get               (){return Regs::OCompareB::Get()            ;}
			static void  EnableInterrupt   (){       Regs::IntMask  ::Or (Regs::OCIE_B);}
			static bool  IsInterrupt       (){return Regs::IntFlag  ::And(Regs::OCIE_B);}
			static void  ClearInterruptFlag(){       Regs::IntFlag  ::Or (Regs::OCF_B );}
		};
		
 }; //.BaseTimerD

// not compiling outside class
//*******************************************************************************
/*
		template<> struct BaseTimerD::Divider <0> { static const ClockDivider value = Div1;    enum {Div = 1   }};
		template<> struct BaseTimerD::Divider <1> { static const ClockDivider value = Div8;    enum {Div = 8   }};
		template<> struct BaseTimerD::Divider <2> { static const ClockDivider value = Div64;   enum {Div = 64  }};
		template<> struct BaseTimerD::Divider <3> { static const ClockDivider value = Div256;  enum {Div = 256 }};
		template<> struct BaseTimerD::Divider <4> { static const ClockDivider value = Div1024; enum {Div = 1024}};
*/

// not compiling outside class

/*	template<>
		class BaseTimerD<>::OutputCompare<0xA> {
	// *******************************************************************************
		public:
			static void  Set      (DataT val){       OCompareA       = val           ;};
			static DataT Get               (){return OCompareA                       ;};
			static void  EnableInterrupt   (){       Regs::IntMask  |= (Regs::OCIE0A);};
			static bool  IsInterrupt       (){return Regs::IntFlag  &  (Regs::OCIE0A);};
			static void  ClearInterruptFlag(){       Regs::IntFlag  |= (Regs::OCF0A );};
 };
		
// not compiling outside class

		template<> 
		class BaseTimerD<>::OutputCompare<0xB> {
	// *******************************************************************************
		public:
			static void  Set      (DataT val){       OCompareB       = val           ;}
			static DataT Get               (){return OCompareB                       ;}
			static void  EnableInterrupt   (){       Regs::IntMask  |= (Regs::OCIE0B);}
			static bool  IsInterrupt       (){return Regs::IntFlag  &  (Regs::OCIE0B);}
			static void  ClearInterruptFlag(){       Regs::IntFlag  |= (Regs::OCF0B );}
		};
*/
//_________________________________________________________________________________
}} //.Timers //.Mcucpp                                                 a.b.08.2015

// Code Snippet
/*********************************************************************************
#define IO_REG_WRAPPER(REG_NAME, CLASS_NAME, DATA_TYPE) \
	struct CLASS_NAME {\
\
		typedef DATA_TYPE DataT;\
\
		static DataT Get  (           ){ return         REG_NAME;         }\
		static void  Set  (DataT value){                REG_NAME  = value;}\
		static void  Or   (DataT value){                REG_NAME |= value;}\
		static void  And  (DataT value){                REG_NAME &= value;}\
		static void  Xor  (DataT value){                REG_NAME ^= value;}\
		static void  AndOr(DataT andMask, DataT orMask){REG_NAME  =(REG_NAME & andMask) | orMask;}\
\
		template<int Bit>\
		static bool  BitIsSet  (){return   REG_NAME & (1 << Bit);}\
\
		template<int Bit>\
		static bool  BitIsClear(){return !(REG_NAME & (1 << Bit));}\
	}

**********************************************************************************/
