//*********************************************************************************
//
// Author		: Konstantin ChizhOV
// Date			: 2013
// All rights reserved.

// Redistribution and use in source and binary forms, with or without modification, 
// are permitted prOVided that the following conditions are met:
// Redistributions of source code must retain the abOVe copyright notice, 
// this list of conditions and the following disclaimer.

// Redistributions in binary form must reproduce the abOVe copyright notice, 
// this list of conditions and the following disclaimer in the documentation and/or 
// other materials prOVided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
// IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
// OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//*********************************************************************************
	#pragma once

	namespace Mcucpp { namespace Timers {
//*********************************************************************************

	template < class Regs // 
//************************
>	class BaseTimerH {
//*********************************************************************************
	public:
		typedef uint16_t DataT;

		enum { 
	//*******************************************************************************
			MaxValue                      = 0xffff 
		};

		enum InterruptFlags {
	//*******************************************************************************
			NoInterrupt                   = 0,
			OverflowInt                   = Regs::TOIE_,
			OutCompare1AInt               = Regs::OCIE_A,
			OutCompare1BInt               = Regs::OCIE_B,
			InCaptureInt                  = Regs::ICIE_
		};

		enum TimerMode {
	//*******************************************************************************
			Normal                        = 0,
			PwmPhaseCorrect8Bit           =                         Regs::WGM_0,
			PwmPhaseCorrect9Bit           =                         Regs::WGM_1,
			PwmPhaseCorrect10Bit          =                         Regs::WGM_1|Regs::WGM_0,
			ClearOnCompareOCR1A           =             Regs::WGM_2,
			PwmFast8Bit                   =             Regs::WGM_2|            Regs::WGM_0,
			PwmFast9Bit                   =             Regs::WGM_2|Regs::WGM_1,
			PwmFast10Bit                  =             Regs::WGM_2|Regs::WGM_1|Regs::WGM_0,
			PwmPhaseAndFreqCorrectToICR1  = Regs::WGM_3, 
			PwmPhaseAndFreqCorrectToOCR1A = Regs::WGM_3|                        Regs::WGM_0,
			PwmPhaseCorrectToICR1         = Regs::WGM_3|            Regs::WGM_1, 
			PwmPhaseCorrectToOCR1A        = Regs::WGM_3|            Regs::WGM_1|Regs::WGM_0,
			ClearOnCompareICR1            = Regs::WGM_3|Regs::WGM_2,
		//Reserved                      = Regs::WGM_3|Regs::WGM_2|            Regs::WGM_0,
			PwmFastToICR1                 = Regs::WGM_3|Regs::WGM_2|Regs::WGM_1, 
			PwmFastToOCR1A                = Regs::WGM_3|Regs::WGM_2|Regs::WGM_1|Regs::WGM_0
		};

		enum ClockDivider {
	//*******************************************************************************
			DivStop                       = 0, 
			Div1                          =                       Regs::CS_0, 
			Div8                          =            Regs::CS_1, 
			Div64                         =            Regs::CS_1|Regs::CS_0, 
			Div256                        = Regs::CS_2,
			Div1024                       = Regs::CS_2|           Regs::CS_0,
			ExtFalling                    = Regs::CS_2|Regs::CS_1, 
			ExtRising                     = Regs::CS_2|Regs::CS_1|Regs::CS_0
		};

		enum { ClockDivider_ClearMask   = ~(Regs::CS_2|Regs::CS_1 |Regs::CS_0 )};
		enum { ControlAMask             =  (           Regs::WGM_1|Regs::WGM_0)};
		enum { ControlBMask             =  (           Regs::WGM_2|Regs::WGM_3)};
		enum { ControlA_ClearMask       = ~(           Regs::WGM_1|Regs::WGM_0)};
		enum { ControlB_ClearMask       = ~(           Regs::WGM_2|Regs::WGM_3)};


		static DataT Get                ( ){return Regs::Counter ::Get()           ;}
		static void  Set        (DataT val){       Regs::Counter ::Set(val)        ;}
		static void  Stop               ( ){       Regs::ControlB::Set(0)          ;}
		static void  Clear              ( ){       Regs::Counter ::Set(0)          ;}
		static void  EnableInterrupt    ( ){       Regs::IntMask ::Or (Regs::TOIE_);}
		static bool  IsInterrupt        ( ){return Regs::IntMask ::And(Regs::TOV_ );}
		static void  ClearInterruptFlag ( ){       Regs::IntMask ::Or (Regs::TOV_ );}

		static void  Start (ClockDivider divider) {
	//*********************************************
			Regs::ControlB::AndOr(ClockDivider_ClearMask,divider);
		}

		static void SetMode(TimerMode mode) {
	//******************************************************************************
			Regs::ControlA::AndOr( ControlA_ClearMask,mode);
			Regs::ControlB::AndOr( ControlB_ClearMask,mode);
		}

		class OutputCompareA {
	// *******************************************************************************
		public:
			static void  Set      (DataT val){       Regs::OCompareA::Set(val)         ;}
			static DataT Get               (){return Regs::OCompareA::Get()            ;}
			static void  EnableInterrupt   (){       Regs::IntMask  ::Or (Regs::OCIE_A);}
			static bool  IsInterrupt       (){return Regs::IntFlag  ::And(Regs::OCIE_A);}
			static void  ClearInterruptFlag(){       Regs::IntFlag  ::Or (Regs::OCF_A );}
 };

		class OutputCompareB {
	// *******************************************************************************
		public:
			static void  Set      (DataT val){       Regs::OCompareB::Set(val)         ;}
			static DataT Get               (){return Regs::OCompareB::Get()            ;}
			static void  EnableInterrupt   (){       Regs::IntMask  ::Or (Regs::OCIE_B);}
			static bool  IsInterrupt       (){return Regs::IntFlag  ::And(Regs::OCIE_B);}
			static void  ClearInterruptFlag(){       Regs::IntFlag  ::Or (Regs::OCF_B );}
		};

//		template<unsigned y> struct Divider;                               // ??
//		template<class Regs,int x> class OutputCompare;

}; //.BaseTimerH

/*
		template<> struct BaseTimerH::Divider <0> { static const ClockDivider value = Div1;    enum {Div = 1   } };
		template<> struct BaseTimerH::Divider <1> { static const ClockDivider value = Div8;    enum {Div = 8   } };
		template<> struct BaseTimerH::Divider <2> { static const ClockDivider value = Div64;   enum {Div = 64  } };
		template<> struct BaseTimerH::Divider <3> { static const ClockDivider value = Div256;  enum {Div = 256 } };
		template<> struct BaseTimerH::Divider <4> { static const ClockDivider value = Div1024; enum {Div = 1024} };
*/
/*
		template<>
		class BaseTimerH<Regs>::OutputCompare<1> {
	// *******************************************************************************
		public:
			static void  Set      (DataT val){       OCompareA = val                 ;}
			static DataT Get               (){return OCompareA                       ;}
			static void  EnableInterrupt   (){       Regs::IntMask  |= (Regs::OCIE_A);}
			static bool  IsInterrupt       (){return Regs::IntFlag  &  (Regs::OCIE_A);}
			static void  ClearInterruptFlag(){       Regs::IntFlag  |= (Regs::OCF_A );}
 };
		
		template<>
		class BaseTimerH<Regs>::OutputCompare<2> {
	// *******************************************************************************
		public:
			static void  Set      (DataT val){       OCompareB = val                 ;}
			static DataT Get               (){return OCompareB                       ;}
			static void  EnableInterrupt   (){       Regs::IntMask  |= (Regs::OCIE_B);}
			static bool  IsInterrupt       (){return Regs::IntFlag  &  (Regs::OCIE_B);}
			static void  ClearInterruptFlag(){       Regs::IntFlag  |= (Regs::OCF_B );}
		};
*/
//_________________________________________________________________________________
}} //.Timers//.Mcucpp                                                  a.b.08.2015
